//
//  UpnpCommand.m
//  uPnp2
//
//  Created by rk on 29/01/11.
//  Copyright 2011 Interchart. All rights reserved.
//

#import "AsynchUpnpCommand.h"
#import	"UpnpAvService.h"


@implementation AsynchUpnpCommand

@synthesize command, params, delegate;

-(XMLTreeNode *)constructSoapEnvelope {
	XMLTreeNode *soapEnvelope = [[XMLTreeNode alloc] init];
	soapEnvelope.name = @"s:Envelope";
	NSMutableDictionary *envelopeAttributes = [[NSMutableDictionary alloc] init];
	[envelopeAttributes setValue:@"http://schemas.xmlsoap.org/soap/envelope/"
						  forKey:@"xmlns:s"];
	[envelopeAttributes setValue:@"http://schemas.xmlsoap.org/soap/encoding/"
						  forKey:@"s:encodingStyle"];
	soapEnvelope.attributes = envelopeAttributes;
	
	return soapEnvelope;
}

-(XMLTreeNode *)constructSoapBody:(UpnpAvService *)aService{
	XMLTreeNode *soapBody = [[XMLTreeNode alloc]init];
	soapBody.name = @"s:Body";
	XMLTreeNode *action = [[XMLTreeNode alloc]init];
	action.name = [NSString stringWithFormat:@"u:%@", self.command];
	NSMutableDictionary *actionAttributes = [[NSMutableDictionary alloc] init];
	[actionAttributes setObject:aService.serviceType forKey:@"xmlns:u"];
	action.attributes = actionAttributes;
	if (self.params) {
		action.children = [[NSMutableDictionary alloc]init];
		NSMutableArray *actionChildList = [[NSMutableArray alloc]init];
		NSArray *keys = [self.params allKeys];
		for (NSString *key in keys) {
			XMLTreeNode *actionParam = [[XMLTreeNode alloc]init];
			actionParam.name = key;
			actionParam.text = [self.params objectForKey:key];
			[actionChildList addObject:actionParam];
		}
		[action.children setObject:actionChildList forKey:self.command];
	}
	action.parent = soapBody;
	
	soapBody.children = [[NSMutableDictionary alloc]init];
	NSMutableArray *childList = [[NSMutableArray alloc]init];
	[childList addObject:action];
	[soapBody.children setObject:childList forKey:action.name];
	
	return soapBody;
}

-(XMLTreeNode *)constructSoapRequest:(UpnpAvService *)aService {
	XMLTreeNode *soapBody = [self constructSoapBody:aService];
	XMLTreeNode *soapRequest = [self constructSoapEnvelope];
	soapBody.parent = soapRequest;
	soapRequest.children = [[NSMutableDictionary alloc]init];
	NSMutableArray *childList = [[NSMutableArray alloc]init];
	[childList addObject:soapBody];
	[soapRequest.children setObject:childList forKey:soapBody.name];	
	
	return soapRequest;	
}

-(void) executeAtUrl:(NSURL *)actionUrl forService:(id)aService {
	UpnpAvService *service = (UpnpAvService *)aService;
	NSLog(@"%@: actionUrl:%@", self, actionUrl);
	NSMutableURLRequest *request = [NSMutableURLRequest 
									requestWithURL:actionUrl 
									cachePolicy:NSURLRequestReloadIgnoringLocalAndRemoteCacheData 
									timeoutInterval:30];
	[request setHTTPMethod:@"POST"];
	[request setValue:
	 [NSString stringWithFormat:@"%@", @"text/xml; charset=\"utf-8\""]
   forHTTPHeaderField:@"Content-Type"];
	[request setValue:
	 [NSString stringWithFormat:@"%@", @"OS/version UPnP/1.0 product/version"]
   forHTTPHeaderField:@"USER-AGENT"];
	NSString *actionName = command;
	NSString *soapAction = [NSString stringWithFormat:@"\"%@#%@\"", 
							service.serviceType,
							actionName];
	[request setValue:soapAction
   forHTTPHeaderField:@"SoapAction"];
	NSString *soapRequestAsString = [[self constructSoapRequest:aService] toXml];
	NSData *soapData = [soapRequestAsString dataUsingEncoding:NSUTF8StringEncoding];
	
	[request setValue:
	 [NSString stringWithFormat:@"%d", [soapData length]]
   forHTTPHeaderField:@"Content-Length"];
	
	[request setHTTPBody:soapData];
	
	NSLog(@"request: %@", request);
	NSLog(@"request method: %@", [request HTTPMethod]);
	NSLog(@"request headers: %@", [request allHTTPHeaderFields]);
	NSLog(@"request body:%.*s", [[request HTTPBody] length], [[request HTTPBody] bytes]);
	[readData release];
	readData = [[NSMutableData alloc] init];
	
	[[NSURLConnection alloc] initWithRequest:request
									delegate:self
							startImmediately:YES];	
	
}


- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error {
	NSLog(@"%@ error: %@", self, error);
}

- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response {
	NSLog(@"%@: response:%@", self, response);
	NSHTTPURLResponse *httpResponse = (NSHTTPURLResponse *)response;
	NSLog(@"  headers: %@", [httpResponse allHeaderFields]);
}
//This method will be called several times as the data arrives
- (void) connection:(NSURLConnection *)connection didReceiveData:(NSData *)data {
	[readData appendData: data];
}

- (void) connectionDidFinishLoading:(NSURLConnection *)connection {
	NSLog(@"readData:'%@', ", [[NSString alloc] initWithData:readData encoding:NSUTF8StringEncoding]);
    NSError *error;
    GDataXMLDocument *doc = [[GDataXMLDocument alloc] initWithData:readData 
                                                           options:0 error:&error];
    NSLog(@"doc.rootElement: %@", doc.rootElement);
	
	if ([delegate respondsToSelector:@selector(handleAsynchUpnpCommandResult:)]) {
		[delegate handleAsynchUpnpCommandResult:doc];
	}
	
    [doc release];

    
	[readData release];
	readData = [[NSMutableData alloc] init]; // ready for next read
	
}


@end
