//
//  ContentDirectoryViewController.m
//  uPnp2
//
//  Created by rk on 10/03/11.
//  Copyright 2011 Interchart. All rights reserved.
//

#import "ContentDirectoryViewController.h"
#import "UpnpAvContentDirectoryService.h"
#import "UpnpAvContentDirectoryContainer.h"
#import "UpnpAvContentDirectoryItem.h"


@implementation ContentDirectoryViewController

@synthesize tableView;

- (id)init {
	self = [super initWithNibName:@"ContentDirectoryView" bundle:nil];
	
	backButtonHistory = [[NSMutableArray alloc] init];
	// These for parent UINavigationController:
	backButtonItem = [[UIBarButtonItem alloc] 
									   initWithTitle:@"Back" 
									   style:UIBarButtonItemStyleBordered 
									   target:self 
									   action:@selector(backButtonPressed:)];
	//[[self navigationItem] setLeftBarButtonItem:backButtonItem];
	[[self navigationItem] setTitle:@"Content Directory"];
	
	return self;
}

-(id)initWithServer:(UpnpAvServer*)theServer {
	self =	[self init];
	[tableView setDelegate:self];
	server = theServer;
	[server.contentDirectoryService getContainerContent:@"0" :self]; // root container has ID @"0"
	[backButtonHistory addObject:@"0"];
	return self;
}

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
	self = [self init];
	return self;
}


-(IBAction)backButtonPressed:(id)sender {
	
	NSString *prevContainerId = nil;
	
	if (backButtonHistory.count > 1) {
		prevContainerId = [backButtonHistory objectAtIndex:(backButtonHistory.count - 2)];
	}
	
	if (prevContainerId) {
		[backButtonHistory removeLastObject];
		[server.contentDirectoryService getContainerContent:prevContainerId :self];
	} 
	if (backButtonHistory.count == 1) { // Only contains root container, hide the back button
		[[self navigationItem] setLeftBarButtonItem:nil];
	}
}


#pragma mark -

-(void)getContainerContentHandler:(NSArray *)theContainers: (NSArray *)theItems {
	containers = theContainers;
	items = theItems;
	for (XMLTreeNode *container in containers) {
		NSLog(@"%@: container:%@", self, container);		
	}
	[tableView setDelegate:self];
	[tableView setDataSource:self];
	[tableView reloadData];
}


#pragma mark -

// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
	[super viewDidLoad];
}



- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
	// Overriden to allow any orientation.
	return YES;
}

#pragma mark -
#pragma mark Table view data source


#define CONTAINERS_SECTION 0
#define ITEMS_SECTION 1
#define NR_SECTIONS 2

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)sectionNr {
	switch(sectionNr) {
		case CONTAINERS_SECTION:
			if (containers.count >0) {
				return [NSString stringWithFormat:@"Folders (%d)", containers.count];
			} else {
				return nil;
			}
			
			break;
		case ITEMS_SECTION:
			if (items.count >0) {
				return [NSString stringWithFormat:@"Items (%d)", items.count];
			} else {
				return nil;
			}
			
			break;
	}
	return @"Other Devices"; // default (fallthru)
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)aTableView {
	// Return the number of sections.
	return NR_SECTIONS;
}


- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
	// Return the number of rows in the section.
	if (section == CONTAINERS_SECTION)
		return containers.count;
	if (section == ITEMS_SECTION)
		return items.count;
	
	return 0; // fallthru
}


// Customize the appearance of table view cells.
- (UITableViewCell *)tableView:(UITableView *)theTableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
	
	static NSString *CellIdentifier = @"Cell";
	
	UITableViewCell *cell = [theTableView dequeueReusableCellWithIdentifier:CellIdentifier];
	if (cell == nil) {
		cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier] autorelease];
	}
	
	if (indexPath.section == CONTAINERS_SECTION) { // Containers section
		XMLTreeNode *containerNode = [containers objectAtIndex:indexPath.row];
		UpnpAvContentDirectoryContainer *container = [[UpnpAvContentDirectoryContainer alloc] initWithXmlTreeNode:containerNode];	
		cell.textLabel.text = [NSString stringWithFormat:@"%@ (%d)", container.title, container.childCount];
	}
	
	if (indexPath.section == ITEMS_SECTION) { // Items section
		XMLTreeNode *itemNode = [items objectAtIndex:indexPath.row];
		UpnpAvContentDirectoryItem *item = [[UpnpAvContentDirectoryItem alloc] initWithXmlTreeNode:itemNode];	
		cell.textLabel.text = [NSString stringWithFormat:@"%@", item.title];
		
	}
	
	return cell;
}


/*
 // Override to support conditional editing of the table view.
 - (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {
 // Return NO if you do not want the specified item to be editable.
 return YES;
 }
 */


/*
 // Override to support editing the table view.
 - (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
 
 if (editingStyle == UITableViewCellEditingStyleDelete) {
 // Delete the row from the data source.
 [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:UITableViewRowAnimationFade];
 }   
 else if (editingStyle == UITableViewCellEditingStyleInsert) {
 // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view.
 }   
 }
 */


/*
 // Override to support rearranging the table view.
 - (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)fromIndexPath toIndexPath:(NSIndexPath *)toIndexPath {
 }
 */


/*
 // Override to support conditional rearranging of the table view.
 - (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath {
 // Return NO if you do not want the item to be re-orderable.
 return YES;
 }
 */


#pragma mark -
#pragma mark Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
	// Navigation logic may go here. Create and push another view controller.
	/*
	 //<#DetailViewController#> *detailViewController = [[<#DetailViewController#> alloc] initWithNibName:@"<#Nib name#>" bundle:nil];
	 // ...
	 // Pass the selected object to the new view controller.
	 [self.navigationController pushViewController:detailViewController animated:YES];
	 [detailViewController release];
	 */
	/*
	 When a row is selected, set the detail view controller's detail item to the item associated with the selected row.
	 */
	//int sectionNr = indexPath.section;
	
	if (indexPath.section == CONTAINERS_SECTION) { // Containers section
		int rowNr = indexPath.row;
		XMLTreeNode *containerNode = [containers objectAtIndex:rowNr];
		UpnpAvContentDirectoryContainer *container = [[UpnpAvContentDirectoryContainer alloc] initWithXmlTreeNode:containerNode];
		NSLog(@"container:%@, title:%@", container, container.title);
		[server.contentDirectoryService getContainerContent:container.id :self];		
		
		[backButtonHistory addObject:container.id];
		[[self navigationItem] setLeftBarButtonItem:backButtonItem];
	}
	if (indexPath.section == ITEMS_SECTION) { // Items section
		int rowNr = indexPath.row;
		XMLTreeNode *itemNode = [items objectAtIndex:rowNr];
		UpnpAvContentDirectoryItem *item = [[UpnpAvContentDirectoryItem alloc] initWithXmlTreeNode:itemNode];
		NSLog(@"item:%@, item.title:%@", item, item.title);
		
		//[server.contentDirectoryService getContainerContent:container.id :self];
	}
	
}

#pragma mark -


- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
	[super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc. that aren't in use.
}


- (void)viewDidUnload {
	[super viewDidUnload];
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
}


- (void)dealloc {
	[super dealloc];
}


@end
