//
//  UpnpAvRenderingControlService.m
//  uPnp2
//
//  Created by rk on 26/01/11.
//  Copyright 2011 Interchart. All rights reserved.
//

#import "UpnpAvRenderingControlService.h"


@implementation UpnpAvRenderingControlService


#pragma mark Designated Initializer
-(id)initWithService:(UpnpAvService *)theService {
	self = [super init];
	service = theService;
	desiredVolume = -1;
	return self;
}

#pragma mark AsynchUpnpCommandDelegate methods
- (void)handleAsynchUpnpCommandResult:(GDataXMLDocument *)parsedResult {
	if ([UpnpAvRenderingControlService isSetVolumeResponse:parsedResult] &&
		volumeChangeIsPending) {
		[self asynchGetVolume:self];
	}
	
	if ([UpnpAvRenderingControlService isGetVolumeResponse:parsedResult]) {
		int currentVolume = [UpnpAvRenderingControlService getVolumeResponseValue:parsedResult];
		if (currentVolume == desiredVolume) {
			volumeChangeIsPending = false; // this method is only called if volume change called
			[desiredVolumeResponseHandler handleAsynchUpnpCommandResult:parsedResult];
			[self asynchGetMute:desiredVolumeResponseHandler];
			
		} else {
			[self sendSetVolumeCommand:desiredVolume];
		}
	}
}


#pragma mark -

- (void) asynchGetMute: (id<AsynchUpnpCommandDelegate>)responseHandler {
	NSLog(@"%@: asynchGetMute", self);
	
	NSString *commandName = @"GetMute";
	NSMutableDictionary *args = [[NSMutableDictionary alloc] init];
	[args setValue:@"0" forKey:@"InstanceID"];
	[args setValue:@"Master" forKey:@"Channel"];
	
	AsynchUpnpCommand *command = [[AsynchUpnpCommand alloc] init];
	command.command = commandName;
	command.params = args;
	command.delegate = responseHandler;
    
	[service executeCommandAsync:command];
}

- (void)asynchSetMute:(BOOL)newValue : (id<AsynchUpnpCommandDelegate>)responseHandler {
	NSLog(@"%@: asynchSetMute", self);
	
	NSString *setValue = @"0";
	if (newValue) setValue = @"1";
	NSString *commandName = @"SetMute";
	NSMutableDictionary *args = [[NSMutableDictionary alloc] init];
	[args setValue:@"0" forKey:@"InstanceID"];
	[args setValue:@"Master" forKey:@"Channel"];
	[args setValue:setValue forKey:@"DesiredMute"];
	
	AsynchUpnpCommand *command = [[AsynchUpnpCommand alloc] init];
	command.command = commandName;
	command.params = args;
	command.delegate = responseHandler;
	
	[service executeCommandAsync:command];
}

- (void)asynchGetVolume:(id<AsynchUpnpCommandDelegate>)responseHandler {
	NSLog(@"%@: asynchGetVolume", self);
	
	NSString *commandName = @"GetVolume";
	NSMutableDictionary *args = [[NSMutableDictionary alloc] init];
	[args setValue:@"0" forKey:@"InstanceID"];
	[args setValue:@"Master" forKey:@"Channel"];
	
	AsynchUpnpCommand *command = [[AsynchUpnpCommand alloc] init];
	command.command = commandName;
	command.params = args;
	command.delegate = responseHandler;
	
	[service executeCommandAsync:command];
}

- (void)asynchSetVolume:(int)newValue : (id<AsynchUpnpCommandDelegate>)responseHandler {
	NSLog(@"%@: asynchSetVolume", self);
	desiredVolume = newValue;
	desiredVolumeResponseHandler = responseHandler;
	
	if (!volumeChangeIsPending) {
		volumeChangeIsPending = true;
		[self sendSetVolumeCommand:desiredVolume];
	}
	
}
#pragma mark Private methods

- (void)sendSetVolumeCommand:(int)newValue {
	NSLog(@"%@: sendSetVolumeCommand", self);
	NSString *setValue = [NSString stringWithFormat:@"%d", desiredVolume];
	NSString *commandName = @"SetVolume";
	NSMutableDictionary *args = [[NSMutableDictionary alloc] init];
	[args setValue:@"0" forKey:@"InstanceID"];
	[args setValue:@"Master" forKey:@"Channel"];
	[args setValue:setValue forKey:@"DesiredVolume"];
	
	AsynchUpnpCommand *command = [[AsynchUpnpCommand alloc] init];
	command.command = commandName;
	command.params = args;
	command.delegate = self;
	
	[service executeCommandAsync:command];
	
	
}

#pragma mark Static Utility Methods
+(bool)isGetMuteResponse:(GDataXMLDocument *)parsedResult {
    NSError *error;
    NSArray *currentMute = [parsedResult nodesForXPath:@"//soap:Envelope/soap:Body/upnpRC:GetMuteResponse/CurrentMute" 
                                            namespaces:[UpnpAvService namespaceDict] 
                                                 error:&error];
	if (currentMute && (currentMute.count > 0)) {
		return true;
	} else {
		return false;
	}
}
+(bool)isSetMuteResponse:(GDataXMLDocument *)parsedResult {
	//if ([parsedResult findChild: @"s:Envelope/s:Body/u:SetMuteResponse"]) {
    NSError *error;
    NSArray *setMuteResponse = [parsedResult nodesForXPath:@"//soap:Envelope/soap:Body/upnpRC:SetMuteResponse" 
                                                namespaces:[UpnpAvService namespaceDict] 
                                                     error:&error];
    if (setMuteResponse && (setMuteResponse.count > 0)) {
		return true;
	} else {
		return false;
	}
}
+(bool)isGetVolumeResponse:(GDataXMLDocument *)parsedResult {
	//if ([parsedResult findChild: @"s:Envelope/s:Body/u:GetVolumeResponse/CurrentVolume"]) {
    NSError *error;
    NSArray *currentVolume = [parsedResult nodesForXPath:@"//soap:Envelope/soap:Body/upnpRC:GetVolumeResponse/CurrentVolume" 
                                              namespaces:[UpnpAvService namespaceDict] 
                                                   error:&error];
	if (currentVolume && (currentVolume.count >0)) {
		return true;
	} else {
		return false;
	}
}
+(bool)isSetVolumeResponse:(GDataXMLDocument *)parsedResult {
	//if ([parsedResult findChild: @"s:Envelope/s:Body/u:SetVolumeResponse"]) {
    NSError *error;
    NSArray *setVolumeResponse = [parsedResult nodesForXPath:@"//soap:Envelope/soap:Body/upnpRC:SetVolumeResponse" 
                                                  namespaces:[UpnpAvService namespaceDict] 
                                                       error:&error];
    if (setVolumeResponse && (setVolumeResponse.count >0)) {
		return true;
	} else {
		return false;
	}
}

+(bool)getMuteResponseValue:(GDataXMLDocument *)parsedResult {
	bool result = false;
	//XMLTreeNode *currentMuteNode = [parsedResult findChild: @"s:Envelope/s:Body/u:GetMuteResponse/CurrentMute"];
    NSError *error;
    NSArray *currentMuteNodes = [parsedResult nodesForXPath:@"//soap:Envelope/soap:Body/upnpRC:GetMuteResponse/CurrentMute" 
                                                 namespaces:[UpnpAvService namespaceDict] 
                                                      error:&error];
    if (currentMuteNodes && (currentMuteNodes.count > 0)) {
        GDataXMLElement *currentMuteNode = (GDataXMLElement *)[currentMuteNodes objectAtIndex:0];
        if (currentMuteNode) {
            int iCurrentMute = currentMuteNode.stringValue.intValue;
            switch (iCurrentMute) {
                case 0:
                    result = false;
                    break;
                case 1:
                    result = true;
                    break;
                default:
                    break;
            }
        }
    }
	return result;
}

// returns -1 if the input is not a response from GetVolume command
+(int)getVolumeResponseValue:(GDataXMLDocument *)parsedResult {
	int result = -1;
    //	XMLTreeNode *currentVolumeNode = [parsedResult findChild: @"s:Envelope/s:Body/u:GetVolumeResponse/CurrentVolume"];
    NSError *error;
    NSArray *currentVolumeNodes = [parsedResult nodesForXPath:@"//soap:Envelope/soap:Body/upnpRC:GetVolumeResponse/CurrentVolume" 
                                                   namespaces:[UpnpAvService namespaceDict] 
                                                        error:&error];
    if (currentVolumeNodes && currentVolumeNodes.count > 0) {
        GDataXMLElement *currentVolumeNode = (GDataXMLElement *)[currentVolumeNodes objectAtIndex:0];
        if (currentVolumeNode) {
            int iCurrentVolume = currentVolumeNode.stringValue.intValue;
            result = iCurrentVolume;
        }
    }
	return result;
}



@end
